<head>
    <style>

        body {
            font-family: "Calibri", sans-serif;
            font-size: 11pt;
        }

        h1 {
            font-weight: bold;
            font-size: 16pt;
            margin: 15px 0px 7px 0px;
        }

        h2 {
            font-weight: bold;
            font-size: 14pt;
            margin: 10px 0px 5px 0px;
        }

        h3 {
            font-weight: bold;
            font-size: 12pt;
            margin: 7px 0px;
        }

        h4 {
            font-weight: bold;
            font-style: italic;
            margin: 5px 0px;
        }

        li {
            line-height: 1.5em;
        }

        ol, ul {
            margin: 5px 0px;
        }

        note {
            vertical-align: super;
            font-size: 0.7em;
            color: #666;
        }

        footnote {
            font-size: 0.9em;
            color: #666;
        }

        code, .code {
            background: #f9f9f9;
            border-left: 4px solid #002060;
            color: #666;
            page-break-inside: avoid;
            font-family: monospace;
            font-size: 0.9em;
            margin: 1.5em 10px;
            max-width: 100%;
            overflow: auto;
            padding: 1em 1.5em;
            display: block;
            word-wrap: break-word;
        }

        blockquote, .blockquote, .quote {
            background: #f9f9f9;
            border-left: 4px solid #ccc;
            margin: 1.5em 10px;
            max-width: 100%;
            overflow: auto;
            padding: 1em 10px .1em;
            quotes: "\201C""\201D""\2018""\2019";
        }

        table, th, td {
            font-size: 10pt;
            padding: 3px 5px;
            border: 1px solid #D9D9D9;
            border-collapse: collapse;
            text-allign: center;
        }

        thead th {
            padding: 5px;
            background-color: #002060;
            color: #FFFFFF;
        }

        tbody th {
            background-color: #F2F2F2;
            text-allign: left;
        }


        .bg_light_grey {background-color: #F2F2F2;}
        .bg_light_purple {background-color: #E4DFEC;}
        .bg_light_blue {background-color: #DCE6F1;}
        .bg_light_green {background-color: #EBF1DE;}
        .bg_light_yellow {background-color: #FFEEB7;}
        .bg_light_orange {background-color: #FDE9D9;}
        .bg_light_red {background-color: #F2DCDB;}

    </style>
</head>
<body>
    {body_html}
</body>
